'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
  MIT License http://www.opensource.org/licenses/mit-license.php
  Author Arun Kumar T K @aruntk
  */
// const loaderUtils = require('loader-utils');
var parse5 = require('parse5');
var polyclean = require('polyclean');
var fs = require('fs');
var path = require('path');
var _ = require('lodash');
var Babel = require('babel-core');
var Synthesizer = require('./synthesis-gen.js');

var DissectHtml = function () {
  function DissectHtml() {
    _classCallCheck(this, DissectHtml);

    this.dissected = {
      js: '//*synthesis*//\n',
      tailJs: '' };
  }

  _createClass(DissectHtml, [{
    key: 'dissect',
    value: function dissect(contents, sourcePath) {
      this.document = contents;
      this.path = sourcePath;
      var self = this;
      var children = this.document.childNodes || [];
      for (var i = 0; i < children.length; i += 1) {
        var child = children[i];
        switch (child.nodeName) {
          case '#documentType':
            break;
          case '#comment':
            break;
          case 'html':
            {
              var _children = child.childNodes || [];
              for (var _i = 0; _i < _children.length; _i += 1) {
                var _child = _children[_i];
                switch (_child.nodeName) {
                  case 'head':
                    {
                      _child.childNodes = self.processChildNodes(_child.childNodes);
                      var headContents = parse5.serialize(_child);
                      // for files inside client folder html contents can be
                      // directly added to dissected.html
                      self.dissected.js += '\n' + Synthesizer.generateJS(headContents, true) + '\n';
                    }
                    break;
                  case 'body':
                    {
                      var body = _child;
                      body.childNodes = self.processChildNodes(body.childNodes);
                      var bodyContents = parse5.serialize(body);
                      self.dissected.js += '\n' + Synthesizer.generateJS(bodyContents) + '\n';
                    }
                    break;
                  default:
                    break;
                }
              }
            }
            break;
          default:
            break;
        }
      }
      this.dissected.js += '\n' + this.dissected.tailJs + '\n';
    }
  }, {
    key: 'processChildNodes',
    value: function processChildNodes(childNodes) {
      var self = this;
      var pushNodes = [];
      var processedNodes = _.compact(_.map(childNodes, function (child) {
        switch (child.nodeName) {
          case 'template':
            {
              var template = child;
              var tmContent = template.content;
              var isWalkable = tmContent && tmContent.nodeName === '#document-fragment' && tmContent.childNodes;
              if (isWalkable) {
                tmContent.childNodes = self.processChildNodes(tmContent.childNodes);
              }
              template.content = tmContent;
              return template;
            }
          case 'link':
            {
              var processedLinkChild = self.processLinks(child);
              if (processedLinkChild) {
                return processedLinkChild;
              }
            }
            break;
          case 'script':
            {
              var result = self.processScripts(child);
              if (result) {
                return result;
              }
            }
            break;
          case 'style':
            {
              if (child.childNodes && child.childNodes.length) {
                var childNode = child.childNodes[0];
                var css = childNode.value;
                var _result = self.processStyle(css);
                if (_result) {
                  childNode.value = _result;
                }
              }
              return child;
            }
          case 'dom-module':
            {
              var domModule = child;
              if (domModule.childNodes) {
                domModule.childNodes = self.processChildNodes(domModule.childNodes);
              }
              return domModule;
            }
          case 'div':
            {
              var attrs = _.filter(child.attrs, function (o) {
                return o.name === 'hidden' || o.name === 'by-vulcanize';
              });
              if (attrs.length >= 2) {
                var _childNodes = self.processChildNodes(child.childNodes);
                pushNodes = pushNodes.concat(_childNodes);
              } else {
                return child;
              }
            }
            break;
          case '#comment':
            break;
          default:
            return child;
        }
        return null;
      }));
      return processedNodes.concat(pushNodes);
    }
  }, {
    key: 'processStyle',
    value: function processStyle(css) {
      return polyclean.stripCss(css);
    }
  }, {
    key: 'processScripts',
    value: function processScripts(child) {
      var self = this;
      var importSource = _.find(child.attrs, function (v) {
        return v.name === 'src';
      });
      if (importSource && importSource.value) {
        var importableUrl = self.importableUrl(importSource.value);
        if (!importableUrl) {
          return child;
        }
        self.dissected.tailJs += '\nrequire(\'' + importableUrl + '\');\n';
      } else {
        self.dissected.tailJs += '\n' + self.babelJs(parse5.serialize(child)) + '\n';
      }
      return null;
    }
  }, {
    key: 'babelJs',
    value: function babelJs(js) {
      // const prod = process.env.NODE_ENV ==='production';
      return Babel.transform(js).code;
    }
  }, {
    key: 'importableUrl',
    value: function importableUrl(url) {
      if (url.match(/^(\/|https?:\/)/)) {
        return false;
      }
      return url.match(/^(\.\/|\.\.\/)/) ? url : './' + url;
    }
  }, {
    key: 'processLinks',
    value: function processLinks(child) {
      var self = this;
      // <link rel='import'...> and <link rel='stylesheet'...>
      var supportedRels = ['import', 'stylesheet'];
      var ifImport = _.find(child.attrs, function (v) {
        return v.name === 'rel' && supportedRels.indexOf(v.value) > -1;
      });
      if (ifImport) {
        var hrefAttr = _.find(child.attrs, function (v) {
          return v.name === 'href';
        });
        if (hrefAttr) {
          if (hrefAttr.value) {
            switch (ifImport.value) {
              case 'import':
                {
                  // file is imported using require
                  var url = self.importableUrl(hrefAttr.value);
                  if (!url) {
                    return child;
                  }
                  var typeAttr = _.find(child.attrs, function (v) {
                    return v.name === 'type';
                  });
                  if (typeAttr) {
                    switch (typeAttr.value) {
                      case 'css':
                        return self.processCssImport(hrefAttr, child);
                      default:
                        break;
                    }
                  }
                  var link = 'require(\'' + url + '\');';
                  self.dissected.tailJs += '\n' + link + '\n';
                }
                break;
              // Processing <link rel='stylesheet' href='filename.css'>
              case 'stylesheet':
                // absolute file path
                return self.processCssImport(hrefAttr, child);
              default:
                break;
            }
          }
        } else {
          return child;
        }
      } else {
        return child;
      }
      return null;
    }
  }, {
    key: 'processCssImport',
    value: function processCssImport(hrefAttr, child) {
      var url = path.resolve(this.path, '../', hrefAttr.value);
      // checks if file exists
      if (fs.existsSync(url)) {
        var contents = fs.readFileSync(url, 'utf8');
        // css is inlined
        var minified = this.processStyle(contents);
        if (minified) {
          // link tag is replaced with style tag
          return _.extend(child, {
            nodeName: 'style',
            tagName: 'style',
            attrs: [],
            childNodes: [{
              nodeName: '#text',
              value: minified
            }]
          });
        }
      }
      return child;
    }
  }]);

  return DissectHtml;
}();

module.exports = function (content) {
  // const query = loaderUtils.parseQuery(this.query);

  if (this.cacheable) {
    this.cacheable();
  }
  // /foo/bar/file.js
  var srcFilepath = this.resourcePath;
  // /foo/bar/file.js -> file
  // const srcFilename = path.basename(srcFilepath, path.extname(srcFilepath));
  // /foo/bar/file.js -> /foo/bar
  // const srcDirpath  = path.dirname(srcFilepath);
  // /foo/bar -> bar
  // const srcDirname  = srcDirpath.split(path.sep).pop();
  var parsed = parse5.parse(content);
  var dissectFn = new DissectHtml();
  dissectFn.dissect(parsed, srcFilepath);
  return dissectFn.dissected.js;
};